@@ -125,16 +125,14 @@ def transitions_masker(
     2 2 3 [1 3 4 5]
     5 1 4 [4 5]
     """
-    n = 0
     pool_items = pool_items.copy()
     pool_output = pool_output.copy()
     if test is not None:
         pool_test = pool_test.copy()
 
-    while n < len(recall_items) - 1:
+    for n in range(len(recall_items) - 1):
         # test if the previous item is in the pool
         if pd.isnull(recall_items[n]) or (recall_items[n] not in pool_items):
-            n += 1
             continue
 
         # remove the item from the pool
@@ -146,7 +144,6 @@ def transitions_masker(
 
         # test if the current item is in the pool
         if pd.isnull(recall_items[n + 1]) or (recall_items[n + 1] not in pool_items):
-            n += 1
             continue
 
         prev = recall_output[n]
@@ -155,7 +152,6 @@ def transitions_masker(
         if test is not None:
             # test if this transition is included
             if not test(recall_test[n], recall_test[n + 1]):
-                n += 1
                 continue
 
             # get included possible items
@@ -163,8 +159,7 @@ def transitions_masker(
             if not isinstance(ind, np.ndarray):
                 ind = np.repeat(ind, poss.shape)
             poss = poss[ind]
-        n += 1
-        yield n, prev, curr, poss
+        yield n + 1, prev, curr, poss
 
 
 def sequences_masker(
@@ -291,6 +286,123 @@ def sequences_masker(
             yield s_output[ind], s_prev[ind], s_curr[ind], s_poss[ind]
 
 
+def windows_masker(
+    list_length,
+    window_lags,
+    pool_items,
+    recall_items,
+    pool_output,
+    recall_output,
+    pool_test=None,
+    recall_test=None,
+    test=None,
+):
+    """
+    Yield windows around previous items in the input list.
+
+    Parameters
+    ----------
+    list_length : int
+        Number of items in each list.
+
+    window_lags : array_like
+        Serial position lags to include in the window.
+
+    pool_items : list
+        Input position of items available for recall.
+
+    recall_items : list
+        Input position of recalled items, in output position order.
+
+    pool_output : list
+        Output values for pool items. Must be the same order as pool.
+
+    recall_output : list
+        Output values in output position order.
+
+    pool_test : list, optional
+        Test values for items available for recall. Must be the same
+        order as pool.
+
+    recall_test : list, optional
+        Test values for items in output position order.
+
+    test : callable, optional
+        Used to test whether individual transitions should be included,
+        based on test values.
+
+            test(prev, curr) - test for included transition
+
+            test(prev, poss) - test for included possible transition
+
+    Yields
+    ------
+    output : int
+        Output positions of included transitions. The first transition
+        is 1.
+
+    prev : list
+        Output values for the "from" item in included transitions.
+
+    curr : list
+        Output values for the "to" item in included transitions.
+
+    poss : list of numpy.ndarray
+        Output values for all possible valid "to" items in included
+        transitions.
+    """
+    # pool items include all items in presentation order; poss items
+    # include only items that have not been recalled yet
+    poss_items = pool_items.copy()
+    poss_output = pool_output.copy()
+    pool_output = np.asarray(pool_output)
+    if test is not None:
+        poss_test = pool_test.copy()
+        pool_test = np.asarray(pool_test)
+    window_lags = np.asarray(window_lags)
+
+    for n in range(len(recall_items) - 1):
+        # test if the previous item is in the pool
+        if pd.isnull(recall_items[n]) or (recall_items[n] not in poss_items):
+            continue
+
+        # remove the item from the pool
+        ind = poss_items.index(recall_items[n])
+        del poss_items[ind]
+        del poss_output[ind]
+        if test is not None:
+            del poss_test[ind]
+
+        # test if the current item is in the pool
+        if pd.isnull(recall_items[n + 1]) or (recall_items[n + 1] not in poss_items):
+            continue
+
+        # get windowed items in the input list
+        prev = int(recall_items[n]) + window_lags
+
+        # exclude if any windowed items do not exist or fail test
+        if np.any(prev < 1) or np.any(prev > list_length):
+            continue
+
+        # exclude current/possible items in the window
+        curr = int(recall_items[n + 1])
+        if curr in prev:
+            continue
+        poss = np.asarray(poss_items, int)
+        include_poss = ~np.isin(poss, prev)
+        poss = poss[include_poss]
+
+        if test is not None:
+            # test if this transition is included
+            if np.any(~test(pool_test[prev - 1], recall_test[n + 1])):
+                continue
+
+            # get included possible items
+            include = test(pool_test[prev - 1][:, np.newaxis], pool_test[poss - 1])
+            poss = poss[np.all(include, axis=0)]
+        yield n + 1, pool_output[prev - 1], pool_output[curr - 1], pool_output[poss - 1]
+
+
 def count_lags(
     list_length,
     pool_items,
@@ -995,6 +1107,88 @@ def rank_distance_shifted(
     return rank
 
 
+def rank_distance_window(
+    distances,
+    list_length,
+    window_lags,
+    pool_items,
+    recall_items,
+    pool_index,
+    recall_index,
+    pool_test=None,
+    recall_test=None,
+    test=None,
+):
+    """
+    Calculate percentile rank of distances relative to items in a window.
+
+    Parameters
+    ----------
+    distances : numpy.array
+        Items x items matrix of pairwise distances or similarities.
+
+    list_length : int
+        Number of items in each list.
+
+    window_lags : array_like
+        Serial position lags to include in the window.
+
+    pool_items : list of list
+        Unique item codes for each item in the pool available for recall.
+
+    recall_items : list of list
+        Unique item codes of recalled items.
+
+    pool_index : list of list
+        Index of each item in the distances matrix.
+
+    recall_index : list of list
+        Index of each recalled item.
+
+    pool_test : list of list, optional
+        Test value for each item in the pool.
+
+    recall_test : list of list, optional
+        Test value for each recalled item.
+
+    test : callable
+        Called as test(prev, curr) or test(prev, poss) to screen
+        actual and possible transitions, respectively.
+
+    Returns
+    -------
+    rank : numpy.ndarray
+        [transitions x window lags] array with distance percentile ranks.
+        The rank is 0 if the distance was the largest of the available
+        transitions, and 1 if the distance was the smallest. Ties are
+        assigned to the average percentile rank.
+    """
+    rank = []
+    for i in range(len(recall_items)):
+        pool_test_list = None if pool_test is None else pool_test[i]
+        recall_test_list = None if recall_test is None else recall_test[i]
+        masker = windows_masker(
+            list_length,
+            window_lags,
+            pool_items[i],
+            recall_items[i],
+            pool_index[i],
+            recall_index[i],
+            pool_test_list,
+            recall_test_list,
+            test,
+        )
+        for output, w_prev, curr, poss in masker:
+            rank_lag = []
+            for prev in w_prev:
+                actual = distances[prev, curr]
+                possible = distances[prev, poss]
+                rank_lag.append(1 - percentile_rank(actual, possible))
+            rank.append(rank_lag)
+    rank = np.array(rank)
+    return rank
+
+
 def count_category(
     pool_items,
     recall_items,
